@@ -165,6 +165,9 @@ class Client(object):
             return False
         return True
 
+    def flush(self):
+        self._transport.flush()
+
     def queue(self, event_type, data, flush=False):
         if not self.should_queue(data):
             return
@@ -182,6 +185,9 @@ class Client(object):
     def instrument(self):
         hoss_agent.instrument()
 
+    def instrument(self):
+        hoss_agent.instrument()
+
     def load_processors(self):
         """
         Loads processors from self.config.processors, as well as constants.HARDCODED_PROCESSORS.
