@@ -42,6 +42,7 @@ from hoss_agent import VERSION
 from hoss_agent.conf import constants
 from hoss_agent.contrib.async_worker import AsyncWorker
 from hoss_agent.utils import compat, json_encoder
+from hoss_agent.utils.compat import BytesIO
 from hoss_agent.utils.logging import get_logger
 from hoss_agent.utils.runtime_env import get_runtime_env
 from hoss_agent.utils.threading import ThreadManager
@@ -171,6 +172,41 @@ class Transport(ThreadManager):
                 max_flush_time = self._max_flush_time * random.uniform(0.9, 1.1) if self._max_flush_time else None
                 self._flushed.set()
 
+    # helper to decode body payload from BytesIO object into string
+    # if fail, payload is removed
+    def _decode_or_truncate_body_payload(self, data):
+        if isinstance(data['request']['body'], bytes):
+            data['request']['body'] = data['request']['body'].decode()
+        elif isinstance(data['request']['body'], BytesIO):
+            data['request']['body'] = data['request']['body'].getvalue().decode()
+        else:
+            logger.debug('Cannot decode request body into string: %s. Removing it')
+            data['request']['body'] = None
+
+        if 'response' in data:
+            if isinstance(data['response']['body'], bytes):
+                data['response']['body'] = data['response']['body'].decode()
+            elif isinstance(data['response']['body'], BytesIO):
+                data['response']['body'] = data['response']['body'].getvalue().decode()
+            else:
+                logger.debug('Cannot decode response body into string: %s. Removing it')
+                data['response']['body'] = None
+
+    # helper to base64 encode request & response payload
+    # if fail, payload is removed
+    def _encode_body_payload(self, data):
+        # encode sanitized payload
+        if isinstance(data['request']['body'], bytes):
+            data['request']['body'] = base64.b64encode(data['request']['body']).decode()
+        else:
+            data['request']['body'] = base64.b64encode(data['request']['body'].encode('utf-8')).decode()
+
+        if 'response' in data:
+            if isinstance(data['response']['body'], bytes):
+                data['response']['body'] = base64.b64encode(data['response']['body']).decode()
+            else:
+                data['response']['body'] = base64.b64encode(data['response']['body'].encode('utf-8')).decode()
+
     def _process_event(self, event_type, data):
         if event_type is None or data is None:
             return data
@@ -181,8 +217,8 @@ class Transport(ThreadManager):
             host = compat.urlparse.urlparse(url).hostname
             api_config = self.client.config.api_host_mapping.get(host)
 
-        data['request']['body'] = data['request']['body'].getvalue().decode()
-        data['response']['body'] = data['response']['body'].getvalue().decode()
+        self._decode_or_truncate_body_payload(data)
+
         # Run the data through processors
         for processor in self._processors:
             if not hasattr(processor, "event_types") or event_type in processor.event_types:
@@ -196,16 +232,7 @@ class Transport(ThreadManager):
                     )
                     return None
 
-        # encode sanitized payload
-        if isinstance(data['request']['body'], bytes):
-            data['request']['body'] = base64.b64encode(data['request']['body']).decode()
-        else:
-            data['request']['body'] = base64.b64encode(data['request']['body'].encode('utf-8')).decode()
-
-        if isinstance(data['response']['body'], bytes):
-            data['response']['body'] = base64.b64encode(data['response']['body']).decode()
-        else:
-            data['response']['body'] = base64.b64encode(data['response']['body'].encode('utf-8')).decode()
+        self._encode_body_payload(data)
 
         data["metadata"] = {
             "platform": get_runtime_env(),
@@ -294,7 +321,7 @@ class Transport(ThreadManager):
             return
         self._closed = True
         self.queue("close", None)
-        if not self._flushed.wait(timeout=self._max_flush_time):
+        if not self._flushed.wait(timeout=self._max_flush_time * 100):
             raise ValueError("close timed out")
 
     stop_thread = close
